@@ -1,4 +1,4 @@
-/* Copyright 2001 by Roger S. Bivand. */
+/* Copyright 2001-8 by Roger S. Bivand. */
 
 #include <R.h>
 #include <Rdefines.h>
@@ -7,11 +7,12 @@
 
 SEXP gsymtest(SEXP nb, SEXP glist, SEXP card)
 {
-	int i, icard, j, k, k1, n=length(nb), pc=0;
-	double g;
+	int i, icard, j, k, k1, n=length(nb), pc=0, l=TRUE;
+	double g, g0, d=0.0, d1=0.0;
 	SEXP ans;
-	PROTECT(ans = NEW_LOGICAL(1)); pc++;
-	LOGICAL_POINTER(ans)[0] = TRUE;
+	PROTECT(ans = NEW_LIST(2)); pc++;
+	SET_VECTOR_ELT(ans, 0, NEW_LOGICAL(1));
+	SET_VECTOR_ELT(ans, 1, NEW_NUMERIC(1));
 
 	for (i=0; i < n; i++) {
 	    icard = INTEGER_POINTER(card)[i];
@@ -22,11 +23,13 @@ SEXP gsymtest(SEXP nb, SEXP glist, SEXP card)
 		    for (k1=0; k1<INTEGER_POINTER(card)[k-ROFFSET]; k1++) {
 			if (i+ROFFSET == INTEGER_POINTER(VECTOR_ELT(nb,
 			    k-ROFFSET))[k1]) {
-			    if (g != NUMERIC_POINTER(VECTOR_ELT(glist,
-			        k-ROFFSET))[k1]) {
-				LOGICAL_POINTER(ans)[0] = FALSE;
-				UNPROTECT(pc);
-				return(ans);
+			    g0 = NUMERIC_POINTER(VECTOR_ELT(glist,
+			        k-ROFFSET))[k1];
+			    d = fabs(g - g0);
+/* Rprintf("%d %d %f %f %f\n", i, j, g, g0, d); */
+			    if (d > 0.0) {
+				l = FALSE;
+				if (d > d1) d1 = d;
 			    }
 			}
 		    }
@@ -34,6 +37,8 @@ SEXP gsymtest(SEXP nb, SEXP glist, SEXP card)
 	    }
 	}
 
+	LOGICAL_POINTER(VECTOR_ELT(ans, 0))[0] = l;
+	NUMERIC_POINTER(VECTOR_ELT(ans, 1))[0] = d1;
 	UNPROTECT(pc); /* ans */
 	return(ans);
 }
